//! # Conversations controller
//!
//! @author Pierre Hubert

use crate::data::http_request_handler::HttpRequestHandler;
use crate::controllers::routes::RequestResult;
use crate::helpers::{user_helper, conversations_helper};
use crate::data::new_conversation::NewConversation;
use crate::api_data::res_create_conversation::ResCreateConversation;
use crate::api_data::conversation_api::ConversationAPI;

/// Create a new conversation
pub fn create(r: &mut HttpRequestHandler) -> RequestResult {
    let name = r.post_string("name")?;
    let mut members = r.post_numbers_list("users", 1)?;

    // Adapt name
    let name = match name.as_str() {
        "false" => None,
        s => Some(s.to_string())
    };

    // Check if members exists
    for user in &members {
        if !user_helper::exists(user.clone())? {
            r.not_found(format!("User {} not found!", user))?;
        }
    }

    // Add current user ID if required
    let curr_user_id = r.user_id()? as i64;
    if !members.contains(&curr_user_id) {
        members.push(curr_user_id);
    }

    let conv = NewConversation {
        owner_id: r.user_id()?,
        name,
        owner_following: r.post_bool("follow")?,
        members,
        can_everyone_add_members: r.post_bool_opt("canEveryoneAddMembers", true),
    };

    // Create the conversation
    let conv_id = conversations_helper::create(&conv)?;
    r.set_response(ResCreateConversation::new(conv_id))
}

/// Get the list of conversations of a user
pub fn get_list(r: &mut HttpRequestHandler) -> RequestResult {
    let list = conversations_helper::get_list_user(r.user_id()?)?;

    r.set_response(list.iter().map(|c| ConversationAPI::new(c)).collect::<Vec<ConversationAPI>>())
}

/// Get information about a single conversation
pub fn get_single(r: &mut HttpRequestHandler) -> RequestResult {
    let conversation_id = r.post_conv_id("conversationID")?;
    let conv = conversations_helper::get_single(conversation_id, r.user_id()?)?;

    r.set_response(ConversationAPI::new(&conv))
}

/// Update the settings of a conversation
pub fn update_settings(r: &mut HttpRequestHandler) -> RequestResult {
    let conv_id = r.post_conv_id("conversationID")?;
    let is_moderator = conversations_helper::is_user_moderator(r.user_id()?, conv_id)?;

    // Update following state, if required
    if r.has_post_parameter("following") {
        conversations_helper::set_following(
            r.user_id()?,
            conv_id,
            r.post_bool("following")?,
        )?;
    }

    // Update members list
    if r.has_post_parameter("members") {
        let mut members = r.post_numbers_list("members", 1)?;
        let can_everyone_add_members = conversations_helper::can_everyone_add_members(conv_id)?;

        if !is_moderator && !can_everyone_add_members {
            r.forbidden("You can not update the list of members of this conversation!".to_string())?;
        }

        if !members.contains(&r.user_id()?) {
            members.push(r.user_id()?);
        }

        conversations_helper::set_members(
            conv_id,
            &members,
            can_everyone_add_members)?;
    }

    r.success("Conversation information successfully updated!")
}